@@ -14,7 +14,6 @@
  * limitations under the License.
  */
 
-// @author: Andrei Alexandrescu (aalexandre)
 // String type.
 
 #pragma once
@@ -28,6 +27,7 @@
 #include <limits>
 #include <stdexcept>
 #include <string>
+#include <string_view>
 #include <type_traits>
 #include <utility>
 
@@ -43,8 +43,8 @@
 #include <folly/lang/Exception.h>
 #include <folly/memory/Malloc.h>
 
-#if FOLLY_HAS_STRING_VIEW
-#include <string_view>
+#if FOLLY_CPLUSPLUS >= 202002L
+#include <compare>
 #endif
 
 FOLLY_PUSH_WARNING
@@ -403,7 +403,8 @@ class fbstring_core {
       auto maybeSmallSize = size_t(maxSmallSize) -
           size_t(static_cast<UChar>(small_[maxSmallSize]));
       // With this syntax, GCC and Clang generate a CMOV instead of a branch.
-      ret = (static_cast<ssize_t>(maybeSmallSize) >= 0) ? maybeSmallSize : ret;
+      ret =
+          (static_cast<ptrdiff_t>(maybeSmallSize) >= 0) ? maybeSmallSize : ret;
     } else {
       ret = (category() == Category::isSmall) ? smallSize() : ret;
     }
@@ -483,7 +484,7 @@ class fbstring_core {
       size_t oldcnt = dis->refCount_.fetch_sub(1, std::memory_order_acq_rel);
       assert(oldcnt > 0);
       if (oldcnt == 1) {
-        free(dis);
+        ::free(dis);
       }
     }
 
@@ -694,13 +695,13 @@ inline void fbstring_core<Char>::initSmall(
     switch ((byteSize + wordWidth - 1) / wordWidth) { // Number of words.
       case 3:
         ml_.capacity_ = reinterpret_cast<const size_t*>(data)[2];
-        FOLLY_FALLTHROUGH;
+        [[fallthrough]];
       case 2:
         ml_.size_ = reinterpret_cast<const size_t*>(data)[1];
-        FOLLY_FALLTHROUGH;
+        [[fallthrough]];
       case 1:
         ml_.data_ = *reinterpret_cast<Char**>(const_cast<Char*>(data));
-        FOLLY_FALLTHROUGH;
+        [[fallthrough]];
       case 0:
         break;
     }
@@ -1011,6 +1012,8 @@ class basic_fbstring {
   typedef std::true_type IsRelocatable;
 
  private:
+  using string_view_type = std::basic_string_view<value_type, traits_type>;
+
   static void procrustes(size_type& n, size_type nmax) {
     if (n > nmax) {
       n = nmax;
@@ -1019,6 +1022,11 @@ class basic_fbstring {
 
   static size_type traitsLength(const value_type* s);
 
+  struct string_view_ctor {};
+  FOLLY_NOINLINE basic_fbstring(
+      string_view_type view, const A&, string_view_ctor)
+      : store_(view.data(), view.size()) {}
+
  public:
   // C++11 21.4.2 construct/copy/destroy
 
@@ -1035,6 +1043,7 @@ class basic_fbstring {
   // these two separate constructors.
 
   basic_fbstring() noexcept : basic_fbstring(A()) {}
+  /* implicit */ basic_fbstring(std::nullptr_t) = delete;
 
   explicit basic_fbstring(const A&) noexcept {}
 
@@ -1098,6 +1107,26 @@ class basic_fbstring {
     assign(il.begin(), il.end());
   }
 
+  template <
+      typename StringViewLike,
+      std::enable_if_t<
+          std::is_convertible_v<const StringViewLike&, string_view_type> &&
+              !std::is_convertible_v<const StringViewLike&, const value_type*>,
+          int> = 0>
+  explicit basic_fbstring(const StringViewLike& view, const A& a = A())
+      : basic_fbstring(string_view_type(view), a, string_view_ctor{}) {}
+
+  template <
+      typename StringViewLike,
+      std::enable_if_t<
+          std::is_convertible_v<const StringViewLike&, string_view_type> &&
+              !std::is_convertible_v<const StringViewLike&, const value_type*>,
+          int> = 0>
+  basic_fbstring(
+      const StringViewLike& view, size_type pos, size_type n, const A& a = A())
+      : basic_fbstring(
+            string_view_type(view).substr(pos, n), a, string_view_ctor{}) {}
+
   ~basic_fbstring() noexcept {}
 
   basic_fbstring& operator=(const basic_fbstring& lhs);
@@ -1116,6 +1145,8 @@ class basic_fbstring {
     return std::basic_string<E, T, A>(data(), size());
   }
 
+  basic_fbstring& operator=(std::nullptr_t) = delete;
+
   basic_fbstring& operator=(const value_type* s) { return assign(s); }
 
   basic_fbstring& operator=(value_type c);
@@ -1143,11 +1174,7 @@ class basic_fbstring {
     return assign(il.begin(), il.end());
   }
 
-#if FOLLY_HAS_STRING_VIEW
-  operator std::basic_string_view<value_type, traits_type>() const noexcept {
-    return {data(), size()};
-  }
-#endif
+  operator string_view_type() const noexcept { return {data(), size()}; }
 
   // C++11 21.4.3 iterators:
   iterator begin() { return store_.mutableData(); }
@@ -1383,7 +1410,8 @@ class basic_fbstring {
  public:
   template <class ItOrLength, class ItOrChar>
   iterator insert(const_iterator p, ItOrLength first_or_n, ItOrChar last_or_c) {
-    using Sel = bool_constant<std::numeric_limits<ItOrLength>::is_specialized>;
+    using Sel =
+        std::bool_constant<std::numeric_limits<ItOrLength>::is_specialized>;
     return insertImplDiscr(p, first_or_n, last_or_c, Sel());
   }
 
@@ -2135,7 +2163,7 @@ inline void basic_fbstring<E, T, A, S>::replaceImpl(
   Invariant checker(*this);
 
   // Handle aliased replace
-  using Sel = bool_constant<
+  using Sel = std::bool_constant<
       std::is_same<FwdIterator, iterator>::value ||
       std::is_same<FwdIterator, const_iterator>::value>;
   if (replaceAliased(i1, i2, s1, s2, Sel())) {
@@ -2408,6 +2436,10 @@ inline bool operator==(
   return lhs.size() == rhs.size() && lhs.compare(rhs) == 0;
 }
 
+template <typename E, class T, class A, class S>
+inline bool operator==(std::nullptr_t, const basic_fbstring<E, T, A, S>&) =
+    delete;
+
 template <typename E, class T, class A, class S>
 inline bool operator==(
     const typename basic_fbstring<E, T, A, S>::value_type* lhs,
@@ -2415,6 +2447,10 @@ inline bool operator==(
   return rhs == lhs;
 }
 
+template <typename E, class T, class A, class S>
+inline bool operator==(const basic_fbstring<E, T, A, S>&, std::nullptr_t) =
+    delete;
+
 template <typename E, class T, class A, class S>
 inline bool operator==(
     const basic_fbstring<E, T, A, S>& lhs,
@@ -2429,6 +2465,10 @@ inline bool operator!=(
   return !(lhs == rhs);
 }
 
+template <typename E, class T, class A, class S>
+inline bool operator!=(std::nullptr_t, const basic_fbstring<E, T, A, S>&) =
+    delete;
+
 template <typename E, class T, class A, class S>
 inline bool operator!=(
     const typename basic_fbstring<E, T, A, S>::value_type* lhs,
@@ -2436,6 +2476,10 @@ inline bool operator!=(
   return !(lhs == rhs);
 }
 
+template <typename E, class T, class A, class S>
+inline bool operator!=(const basic_fbstring<E, T, A, S>&, std::nullptr_t) =
+    delete;
+
 template <typename E, class T, class A, class S>
 inline bool operator!=(
     const basic_fbstring<E, T, A, S>& lhs,
@@ -2450,6 +2494,10 @@ inline bool operator<(
   return lhs.compare(rhs) < 0;
 }
 
+template <typename E, class T, class A, class S>
+inline bool operator<(const basic_fbstring<E, T, A, S>&, std::nullptr_t) =
+    delete;
+
 template <typename E, class T, class A, class S>
 inline bool operator<(
     const basic_fbstring<E, T, A, S>& lhs,
@@ -2457,6 +2505,10 @@ inline bool operator<(
   return lhs.compare(rhs) < 0;
 }
 
+template <typename E, class T, class A, class S>
+inline bool operator<(std::nullptr_t, const basic_fbstring<E, T, A, S>&) =
+    delete;
+
 template <typename E, class T, class A, class S>
 inline bool operator<(
     const typename basic_fbstring<E, T, A, S>::value_type* lhs,
@@ -2471,6 +2523,10 @@ inline bool operator>(
   return rhs < lhs;
 }
 
+template <typename E, class T, class A, class S>
+inline bool operator>(const basic_fbstring<E, T, A, S>&, std::nullptr_t) =
+    delete;
+
 template <typename E, class T, class A, class S>
 inline bool operator>(
     const basic_fbstring<E, T, A, S>& lhs,
@@ -2478,6 +2534,10 @@ inline bool operator>(
   return rhs < lhs;
 }
 
+template <typename E, class T, class A, class S>
+inline bool operator>(std::nullptr_t, const basic_fbstring<E, T, A, S>&) =
+    delete;
+
 template <typename E, class T, class A, class S>
 inline bool operator>(
     const typename basic_fbstring<E, T, A, S>::value_type* lhs,
@@ -2492,6 +2552,10 @@ inline bool operator<=(
   return !(rhs < lhs);
 }
 
+template <typename E, class T, class A, class S>
+inline bool operator<=(const basic_fbstring<E, T, A, S>&, std::nullptr_t) =
+    delete;
+
 template <typename E, class T, class A, class S>
 inline bool operator<=(
     const basic_fbstring<E, T, A, S>& lhs,
@@ -2499,6 +2563,10 @@ inline bool operator<=(
   return !(rhs < lhs);
 }
 
+template <typename E, class T, class A, class S>
+inline bool operator<=(std::nullptr_t, const basic_fbstring<E, T, A, S>&) =
+    delete;
+
 template <typename E, class T, class A, class S>
 inline bool operator<=(
     const typename basic_fbstring<E, T, A, S>::value_type* lhs,
@@ -2513,6 +2581,10 @@ inline bool operator>=(
   return !(lhs < rhs);
 }
 
+template <typename E, class T, class A, class S>
+inline bool operator>=(const basic_fbstring<E, T, A, S>&, std::nullptr_t) =
+    delete;
+
 template <typename E, class T, class A, class S>
 inline bool operator>=(
     const basic_fbstring<E, T, A, S>& lhs,
@@ -2520,6 +2592,10 @@ inline bool operator>=(
   return !(lhs < rhs);
 }
 
+template <typename E, class T, class A, class S>
+inline bool operator>=(std::nullptr_t, const basic_fbstring<E, T, A, S>&) =
+    delete;
+
 template <typename E, class T, class A, class S>
 inline bool operator>=(
     const typename basic_fbstring<E, T, A, S>::value_type* lhs,
@@ -2527,6 +2603,16 @@ inline bool operator>=(
   return !(lhs < rhs);
 }
 
+#if FOLLY_CPLUSPLUS >= 202002
+template <typename E, class T, class A, class S>
+inline bool operator<=>(std::nullptr_t, const basic_fbstring<E, T, A, S>&) =
+    delete;
+
+template <typename E, class T, class A, class S>
+inline bool operator<=>(const basic_fbstring<E, T, A, S>&, std::nullptr_t) =
+    delete;
+#endif
+
 // C++11 21.4.8.8
 template <typename E, class T, class A, class S>
 void swap(basic_fbstring<E, T, A, S>& lhs, basic_fbstring<E, T, A, S>& rhs) {
@@ -2622,12 +2708,6 @@ operator<<(
   return os;
 }
 
-#if FOLLY_CPLUSPLUS < 201703L
-template <typename E1, class T, class A, class S>
-constexpr typename basic_fbstring<E1, T, A, S>::size_type
-    basic_fbstring<E1, T, A, S>::npos;
-#endif
-
 // basic_string compatibility routines
 
 template <typename E, class T, class A, class S, class A2>
