@@ -22,13 +22,13 @@
 #endif
 #include <fstream>
 #include <mutex>
+#include <numeric>
+#include <optional>
 
 #include <fmt/core.h>
-
 #include <glog/logging.h>
 #include <folly/Indestructible.h>
 #include <folly/Memory.h>
-#include <folly/Optional.h>
 #include <folly/ScopeGuard.h>
 #include <folly/detail/StaticSingletonManager.h>
 #include <folly/hash/Hash.h>
@@ -80,25 +80,77 @@ const CacheLocality& CacheLocality::system<std::atomic>() {
   return *value;
 }
 
-// Each level of cache has sharing sets, which are the set of cpus
-// that share a common cache at that level.  These are available in a
-// hex bitset form (/sys/devices/system/cpu/cpu0/index0/shared_cpu_map,
-// for example).  They are also available in a human-readable list form,
-// as in /sys/devices/system/cpu/cpu0/index0/shared_cpu_list.  The list
-// is a comma-separated list of numbers and ranges, where the ranges are
-// a pair of decimal numbers separated by a '-'.
-//
-// To sort the cpus for optimum locality we don't really need to parse
-// the sharing sets, we just need a unique representative from the
-// equivalence class.  The smallest value works fine, and happens to be
-// the first decimal number in the file.  We load all of the equivalence
-// class information from all of the cpu*/index* directories, order the
-// cpus first by increasing last-level cache equivalence class, then by
-// the smaller caches.  Finally, we break ties with the cpu number itself.
+CacheLocality::CacheLocality(std::vector<std::vector<size_t>> equivClasses) {
+  numCpus = equivClasses.size();
 
-/// Returns the first decimal number in the string, or throws an exception
-/// if the string does not start with a number terminated by ',', '-',
-/// '\n', or eos.
+  for (size_t cpu = 0; cpu < numCpus; ++cpu) {
+    for (size_t level = 0; level < equivClasses[cpu].size(); ++level) {
+      if (equivClasses[cpu][level] == cpu) {
+        // we only want to count the equiv classes once, so we do it when we
+        // are processing their representative.
+        while (numCachesByLevel.size() <= level) {
+          numCachesByLevel.push_back(0);
+        }
+        numCachesByLevel[level]++;
+      }
+    }
+  }
+
+  std::vector<size_t> cpus(numCpus);
+  std::iota(cpus.begin(), cpus.end(), 0);
+
+  std::sort(cpus.begin(), cpus.end(), [&](size_t lhs, size_t rhs) -> bool {
+    auto& lhsEquiv = equivClasses[lhs];
+    auto& rhsEquiv = equivClasses[rhs];
+
+    // If different cpus have different numbers of caches group first by number
+    // of caches to guarantee strict weak ordering, even though the resulting
+    // order may be sub-optimal.
+    if (lhsEquiv.size() != rhsEquiv.size()) {
+      return lhsEquiv.size() < rhsEquiv.size();
+    }
+
+    // Order by equiv class of cache with highest index, direction doesn't
+    // matter.
+    for (size_t i = lhsEquiv.size(); i > 0; --i) {
+      auto idx = i - 1;
+      if (lhsEquiv[idx] != rhsEquiv[idx]) {
+        return lhsEquiv[idx] < rhsEquiv[idx];
+      }
+    }
+
+    // Break ties deterministically by cpu.
+    return lhs < rhs;
+  });
+
+  // The cpus are now sorted by locality, with neighboring entries closer
+  // to each other than entries that are far away.  For striping we want
+  // the inverse map, since we are starting with the cpu.
+  localityIndexByCpu.resize(numCpus);
+  for (size_t i = 0; i < cpus.size(); ++i) {
+    localityIndexByCpu[cpus[i]] = i;
+  }
+
+  equivClassesByCpu = std::move(equivClasses);
+}
+
+// Each level of cache has sharing sets, which are the set of cpus that share a
+// common cache at that level.  These are available in a hex bitset form
+// (/sys/devices/system/cpu/cpu0/cache/index0/shared_cpu_map, for example).
+// They are also available in human-readable form in the shared_cpu_list file in
+// the same directory.  The list is a comma-separated list of numbers and
+// ranges, where the ranges are pairs of decimal numbers separated by a '-'.
+//
+// To sort the cpus for optimum locality we don't really need to parse the
+// sharing sets, we just need a unique representative from the equivalence
+// class.  The smallest value works fine, and happens to be the first decimal
+// number in the file.  We load all of the equivalence class information from
+// all of the cpu*/index* directories, order the cpus first by increasing
+// last-level cache equivalence class, then by the smaller caches.  Finally, we
+// break ties with the cpu number itself.
+
+/// Returns the first decimal number in the line, or throws an exception if the
+/// line does not start with a number terminated by ',', '-', '\n', or EOS.
 static size_t parseLeadingNumber(const std::string& line) {
   auto raw = line.c_str();
   char* end;
@@ -110,18 +162,12 @@ static size_t parseLeadingNumber(const std::string& line) {
 }
 
 CacheLocality CacheLocality::readFromSysfsTree(
-    const std::function<std::string(std::string)>& mapping) {
-  // number of equivalence classes per level
-  std::vector<size_t> numCachesByLevel;
-
-  // the list of cache equivalence classes, where equivalance classes
+    const std::function<std::string(std::string const&)>& mapping) {
+  // the list of cache equivalence classes, where equivalence classes
   // are named by the smallest cpu in the class
   std::vector<std::vector<size_t>> equivClassesByCpu;
 
-  std::vector<size_t> cpus;
-
-  while (true) {
-    auto cpu = cpus.size();
+  for (size_t cpu = 0;; ++cpu) {
     std::vector<size_t> levels;
     for (size_t index = 0;; ++index) {
       auto dir = fmt::format(
@@ -137,17 +183,7 @@ CacheLocality CacheLocality::readFromSysfsTree(
         continue;
       }
       auto equiv = parseLeadingNumber(equivStr);
-      auto level = levels.size();
       levels.push_back(equiv);
-
-      if (equiv == cpu) {
-        // we only want to count the equiv classes once, so we do it when
-        // we first encounter them
-        while (numCachesByLevel.size() <= level) {
-          numCachesByLevel.push_back(0);
-        }
-        numCachesByLevel[level]++;
-      }
     }
 
     if (levels.empty()) {
@@ -155,47 +191,17 @@ CacheLocality CacheLocality::readFromSysfsTree(
       break;
     }
     equivClassesByCpu.emplace_back(std::move(levels));
-    cpus.push_back(cpu);
   }
 
-  if (cpus.empty()) {
+  if (equivClassesByCpu.empty()) {
     throw std::runtime_error("unable to load cache sharing info");
   }
 
-  std::sort(cpus.begin(), cpus.end(), [&](size_t lhs, size_t rhs) -> bool {
-    // sort first by equiv class of cache with highest index,
-    // direction doesn't matter.  If different cpus have
-    // different numbers of caches then this code might produce
-    // a sub-optimal ordering, but it won't crash
-    auto& lhsEquiv = equivClassesByCpu[lhs];
-    auto& rhsEquiv = equivClassesByCpu[rhs];
-    for (ssize_t i = ssize_t(std::min(lhsEquiv.size(), rhsEquiv.size())) - 1;
-         i >= 0;
-         --i) {
-      auto idx = size_t(i);
-      if (lhsEquiv[idx] != rhsEquiv[idx]) {
-        return lhsEquiv[idx] < rhsEquiv[idx];
-      }
-    }
-
-    // break ties deterministically by cpu
-    return lhs < rhs;
-  });
-
-  // the cpus are now sorted by locality, with neighboring entries closer
-  // to each other than entries that are far away.  For striping we want
-  // the inverse map, since we are starting with the cpu
-  std::vector<size_t> indexes(cpus.size());
-  for (size_t i = 0; i < cpus.size(); ++i) {
-    indexes[cpus[i]] = i;
-  }
-
-  return CacheLocality{
-      cpus.size(), std::move(numCachesByLevel), std::move(indexes)};
+  return CacheLocality{std::move(equivClassesByCpu)};
 }
 
 CacheLocality CacheLocality::readFromSysfs() {
-  return readFromSysfsTree([](std::string name) {
+  return readFromSysfsTree([](std::string const& name) {
     std::ifstream xi(name.c_str());
     std::string rv;
     std::getline(xi, rv);
@@ -203,15 +209,17 @@ CacheLocality CacheLocality::readFromSysfs() {
   });
 }
 
+namespace {
+
 static bool procCpuinfoLineRelevant(std::string const& line) {
   return line.size() > 4 && (line[0] == 'p' || line[0] == 'c');
 }
 
-CacheLocality CacheLocality::readFromProcCpuinfoLines(
+std::vector<std::tuple<size_t, size_t, size_t>> parseProcCpuinfoLines(
     std::vector<std::string> const& lines) {
+  std::vector<std::tuple<size_t, size_t, size_t>> cpus;
   size_t physicalId = 0;
   size_t coreId = 0;
-  std::vector<std::tuple<size_t, size_t, size_t>> cpus;
   size_t maxCpu = 0;
   for (auto iter = lines.rbegin(); iter != lines.rend(); ++iter) {
     auto& line = *iter;
@@ -249,29 +257,40 @@ CacheLocality CacheLocality::readFromProcCpuinfoLines(
         "offline CPUs not supported for /proc/cpuinfo cache locality source");
   }
 
+  return cpus;
+}
+
+} // namespace
+
+CacheLocality CacheLocality::readFromProcCpuinfoLines(
+    std::vector<std::string> const& lines) {
+  // (physicalId, coreId, cpu)
+  std::vector<std::tuple<size_t, size_t, size_t>> cpus =
+      parseProcCpuinfoLines(lines);
+  // Sort to make equivalence classes contiguous.
   std::sort(cpus.begin(), cpus.end());
-  size_t cpusPerCore = 1;
-  while (cpusPerCore < cpus.size() &&
-         std::get<0>(cpus[cpusPerCore]) == std::get<0>(cpus[0]) &&
-         std::get<1>(cpus[cpusPerCore]) == std::get<1>(cpus[0])) {
-    ++cpusPerCore;
-  }
 
-  // we can't tell the real cache hierarchy from /proc/cpuinfo, but it
-  // works well enough to assume there are 3 levels, L1 and L2 per-core
-  // and L3 per socket
-  std::vector<size_t> numCachesByLevel;
-  numCachesByLevel.push_back(cpus.size() / cpusPerCore);
-  numCachesByLevel.push_back(cpus.size() / cpusPerCore);
-  numCachesByLevel.push_back(std::get<0>(cpus.back()) + 1);
-
-  std::vector<size_t> indexes(cpus.size());
+  // We can't tell the real cache hierarchy from /proc/cpuinfo, but it works
+  // well enough to assume there are 3 levels, L1 and L2 per-core and L3 per
+  // socket.
+  std::vector<std::vector<size_t>> equivClassesByCpu(cpus.size());
+  size_t l1Equiv = 0;
+  size_t l3Equiv = 0;
   for (size_t i = 0; i < cpus.size(); ++i) {
-    indexes[std::get<2>(cpus[i])] = i;
+    auto [physicalId, coreId, cpu] = cpus[i];
+    // The representative for each L1 and L3 equivalence class is the first cpu
+    // in the class.
+    if (i == 0 || physicalId != std::get<0>(cpus[i - 1]) ||
+        coreId != std::get<1>(cpus[i - 1])) {
+      l1Equiv = cpu;
+    }
+    if (i == 0 || physicalId != std::get<0>(cpus[i - 1])) {
+      l3Equiv = cpu;
+    }
+    equivClassesByCpu[cpu] = {l1Equiv, l1Equiv, l3Equiv};
   }
 
-  return CacheLocality{
-      cpus.size(), std::move(numCachesByLevel), std::move(indexes)};
+  return CacheLocality{std::move(equivClassesByCpu)};
 }
 
 CacheLocality CacheLocality::readFromProcCpuinfo() {
@@ -294,19 +313,9 @@ CacheLocality CacheLocality::readFromProcCpuinfo() {
 }
 
 CacheLocality CacheLocality::uniform(size_t numCpus) {
-  CacheLocality rv;
-
-  rv.numCpus = numCpus;
-
-  // one cache shared by all cpus
-  rv.numCachesByLevel.push_back(numCpus);
-
-  // no permutations in locality index mapping
-  for (size_t cpu = 0; cpu < numCpus; ++cpu) {
-    rv.localityIndexByCpu.push_back(cpu);
-  }
-
-  return rv;
+  // One cache shared by all cpus.
+  std::vector<std::vector<size_t>> equivClassesByCpu(numCpus, {0});
+  return CacheLocality{std::move(equivClassesByCpu)};
 }
 
 ////////////// Getcpu
@@ -541,7 +550,7 @@ class Allocator {
   }
 
  private:
-  folly::Optional<uint8_t> sizeClass(size_t size) {
+  std::optional<uint8_t> sizeClass(size_t size) {
     if (size <= 8) {
       return 0;
     } else if (size <= 16) {
@@ -551,7 +560,7 @@ class Allocator {
     } else if (size <= 64) {
       return 3;
     } else {
-      return folly::none;
+      return std::nullopt;
     }
   }
 
