@@ -20,6 +20,7 @@
 
 #include <atomic>
 #include <functional>
+#include <memory>
 #include <mutex>
 #include <string>
 #include <thread>
@@ -57,6 +58,41 @@ namespace threadlocal_detail {
 
 constexpr uint32_t kEntryIDInvalid = std::numeric_limits<uint32_t>::max();
 
+//  as a memory-usage optimization, try to make this deleter fit in-situ in
+//  the deleter function storage rather than being heap-allocated separately
+//
+//  for libstdc++, specialization below of std::__is_location_invariant
+//
+//  TODO: ensure in-situ storage for other standard-library implementations
+struct SharedPtrDeleter {
+  mutable std::shared_ptr<void> ts_;
+  explicit SharedPtrDeleter(std::shared_ptr<void> const& ts) noexcept;
+  SharedPtrDeleter(SharedPtrDeleter const& that) noexcept;
+  void operator=(SharedPtrDeleter const& that) = delete;
+  ~SharedPtrDeleter();
+  void operator()(void* ptr, folly::TLPDestructionMode) const;
+};
+
+} // namespace threadlocal_detail
+
+} // namespace folly
+
+#if defined(__GLIBCXX__)
+
+namespace std {
+
+template <>
+struct __is_location_invariant<::folly::threadlocal_detail::SharedPtrDeleter>
+    : std::true_type {};
+
+} // namespace std
+
+#endif
+
+namespace folly {
+
+namespace threadlocal_detail {
+
 /**
  * POD wrapper around an element (a void*) and an associated deleter.
  * This must be POD, as we memset() it to 0 and memcpy() it around.
@@ -72,6 +108,9 @@ struct ElementWrapper {
 
   static_assert(alignof(DeleterObjType) > deleter_all_mask);
 
+  //  must be noinline and must launder: https://godbolt.org/z/bo6f7f6v6
+  FOLLY_NOINLINE static uintptr_t castForgetAlign(DeleterFunType*) noexcept;
+
   bool dispose(TLPDestructionMode mode) noexcept {
     if (ptr == nullptr) {
       return false;
@@ -109,7 +148,7 @@ struct ElementWrapper {
     }
     auto const fun =
         +[](void* pt, TLPDestructionMode) { delete static_cast<Ptr>(pt); };
-    auto const raw = reinterpret_cast<uintptr_t>(fun);
+    auto const raw = castForgetAlign(fun);
     if (raw & deleter_all_mask) {
       return set(p, std::ref(*fun));
     }
@@ -118,6 +157,18 @@ struct ElementWrapper {
     ptr = p;
   }
 
+  template <typename Ptr, typename Deleter>
+  static auto makeDeleter(const Deleter& d) {
+    return [d](void* pt, TLPDestructionMode mode) {
+      d(static_cast<Ptr>(pt), mode);
+    };
+  }
+
+  template <typename Ptr>
+  static decltype(auto) makeDeleter(const SharedPtrDeleter& d) {
+    return d;
+  }
+
   template <class Ptr, class Deleter>
   void set(Ptr p, const Deleter& d) {
     DCHECK_EQ(static_cast<void*>(nullptr), ptr);
@@ -128,9 +179,7 @@ struct ElementWrapper {
     }
 
     auto guard = makeGuard([&] { d(p, TLPDestructionMode::THIS_THREAD); });
-    auto const obj = new DeleterObjType([d](void* pt, TLPDestructionMode mode) {
-      d(static_cast<Ptr>(pt), mode);
-    });
+    auto const obj = new DeleterObjType(makeDeleter<Ptr>(d));
     guard.dismiss();
     auto const raw = reinterpret_cast<uintptr_t>(obj);
     DCHECK_EQ(0, raw & deleter_all_mask);
