@@ -36,6 +36,18 @@ namespace folly {
 
 const size_t CPUThreadPoolExecutor::kDefaultMaxQueueSize = 1 << 14;
 
+CPUThreadPoolExecutor::CPUTask::CPUTask(
+    Func&& f,
+    std::chrono::milliseconds expiration,
+    Func&& expireCallback,
+    int8_t pri)
+    : Task(std::move(f), expiration, std::move(expireCallback), pri) {
+  DCHECK(func_); // Empty func reserved as poison.
+}
+
+CPUThreadPoolExecutor::CPUTask::CPUTask()
+    : Task(nullptr, std::chrono::milliseconds(0), nullptr) {}
+
 /* static */ auto CPUThreadPoolExecutor::makeDefaultQueue()
     -> std::unique_ptr<BlockingQueue<CPUTask>> {
   return std::make_unique<UnboundedBlockingQueue<CPUTask>>();
@@ -87,9 +99,6 @@ CPUThreadPoolExecutor::CPUThreadPoolExecutor(
     : ThreadPoolExecutor(
           numThreads.first, numThreads.second, std::move(threadFactory)),
       taskQueue_(taskQueue.release()),
-      queueObserverFactory_{
-          !opt.queueObserverFactory ? createQueueObserverFactory()
-                                    : std::move(opt.queueObserverFactory)},
       prohibitBlockingOnThreadPools_{opt.blocking} {
   setNumThreads(numThreads.first);
   if (numThreads.second == 0) {
@@ -115,9 +124,6 @@ CPUThreadPoolExecutor::CPUThreadPoolExecutor(
     : ThreadPoolExecutor(
           numThreads.first, numThreads.second, std::move(threadFactory)),
       taskQueue_(makeDefaultQueue()),
-      queueObserverFactory_{
-          opt.queueObserverFactory ? std::move(opt.queueObserverFactory)
-                                   : createQueueObserverFactory()},
       prohibitBlockingOnThreadPools_{opt.blocking} {
   setNumThreads(numThreads.first);
   if (numThreads.second == 0) {
@@ -223,15 +229,22 @@ void CPUThreadPoolExecutor::addImpl(
     int8_t priority,
     std::chrono::milliseconds expiration,
     Func expireCallback) {
-  if (withPriority) {
-    CHECK(getNumPriorities() > 0);
+  if (!func) {
+    // Reserve empty funcs as poison by logging the error inline.
+    invokeCatchingExns("ThreadPoolExecutor: func", std::move(func));
+    return;
   }
+
+  if (withPriority) {
+    CHECK_GT(getNumPriorities(), 0);
+  }
+
   CPUTask task(
       std::move(func), expiration, std::move(expireCallback), priority);
   if (auto queueObserver = getQueueObserver(priority)) {
-    task.queueObserverPayload() =
-        queueObserver->onEnqueued(task.context_.get());
+    task.queueObserverPayload_ = queueObserver->onEnqueued(task.context_.get());
   }
+  registerTaskEnqueue(task);
 
   // It's not safe to expect that the executor is alive after a task is added to
   // the queue (this task could be holding the last KeepAlive and when finished
@@ -262,6 +275,10 @@ size_t CPUThreadPoolExecutor::getTaskQueueSize() const {
   return taskQueue_->size();
 }
 
+WorkerProvider* CPUThreadPoolExecutor::getThreadIdCollector() {
+  return threadIdCollector_.get();
+}
+
 BlockingQueue<CPUThreadPoolExecutor::CPUTask>*
 CPUThreadPoolExecutor::getTaskQueue() {
   return taskQueue_.get();
@@ -312,7 +329,7 @@ void CPUThreadPoolExecutor::threadRun(ThreadPtr thread) {
 
     // Handle thread stopping, either by task timeout, or
     // by 'poison' task added in join() or stop().
-    if (FOLLY_UNLIKELY(!task || task.value().poison)) {
+    if (FOLLY_UNLIKELY(!task || !task->func_)) {
       // Actually remove the thread from the list.
       std::unique_lock w{threadListLock_};
       if (taskShouldStop(task)) {
@@ -327,8 +344,8 @@ void CPUThreadPoolExecutor::threadRun(ThreadPtr thread) {
       }
     }
 
-    if (auto queueObserver = getQueueObserver(task->queuePriority())) {
-      queueObserver->onDequeued(task->queueObserverPayload());
+    if (auto queueObserver = getQueueObserver(task->priority())) {
+      queueObserver->onDequeued(task->queueObserverPayload_);
     }
     runTask(thread, std::move(task.value()));
 
