@@ -23,10 +23,10 @@
 #include <folly/File.h>
 #include <folly/String.h>
 #include <folly/Subprocess.h>
-#include <folly/experimental/TestUtil.h>
 #include <folly/experimental/io/FsUtil.h>
 #include <folly/portability/GFlags.h>
 #include <folly/portability/GTest.h>
+#include <folly/testing/TestUtil.h>
 
 using namespace folly;
 using namespace folly::test;
